package org.objectweb.telosys.tunit;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.auth.Auth;
import org.objectweb.telosys.auth.IAppUser;
import org.objectweb.telosys.auth.LoginUser;
import org.objectweb.telosys.common.IExternalMainFactory;
import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.dal.TelosysDAL;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;


public class ActionTestManager 
{
	private FakeHttpSessionManager sessionManager = null ;
	
	/**
	 * Constructor <br>
	 * Initialize the Telosys environment, including the Data Access Layer <br>
	 * and create a new "session manager" ( all the requests managed by a <br> 
	 * test manager are associated with the same session )
	 * 
	 * @param extFactory the 'external factory' used in the application 
	 * @param telosysProperties the 'telosys.properties' file path 
	 */
	public ActionTestManager(IExternalMainFactory extFactory, String telosysProperties)
	{
        //--- 1) Init Telosys env
        Telosys.init(extFactory, telosysProperties );
        
    	//--- 2) Init Database connection provider 
    	//TelosysDAL.init(DB_CONFIG);
    	TelosysDAL.init();
        
    	//--- 3) Init the HttpSession manager
    	sessionManager = initSessionManager() ;
    			
	}

	//-------------------------------------------------------------------------------------
    /**
     * Creates and return a new ScreenRequest instance
     * @return
     */
    public ScreenRequest newScreenRequest()
    {
    	HttpServletRequest httpServletRequest = new FakeHttpServletRequest(sessionManager);
    	
    	ScreenRequest screenRequest = new ScreenRequest( httpServletRequest ) ;
    	return screenRequest ;
    }
    
	//-------------------------------------------------------------------------------------
    /**
     * Creates and return a new ScreenRequest instance, 
     * with 'screen name' and 'screen id' initialization
     * @param screenName
     * @param screenId
     * @return
     */
    public ScreenRequest newScreenRequest(String screenName, int screenId )
    {
    	HttpServletRequest httpServletRequest = new FakeHttpServletRequest(sessionManager);
    	
    	ScreenRequest screenRequest = new ScreenRequest( httpServletRequest ) ;
    	screenRequest.setScreenName(screenName);
    	screenRequest.setScreenId(""+screenId);
    	return screenRequest ;
    }
    
	//-------------------------------------------------------------------------------------
    /**
     * Creates and return a new ScreenRequest instance, 
     * with 'screen name', 'screen id' and 'action' initialization
     * @param screenName
     * @param screenId
     * @param action
     * @return
     */
    public ScreenRequest newScreenRequest(String screenName, int screenId, String action )
    {
    	HttpServletRequest httpServletRequest = new FakeHttpServletRequest(sessionManager);
    	
    	ScreenRequest screenRequest = new ScreenRequest( httpServletRequest ) ;
    	screenRequest.setScreenName(screenName);
    	screenRequest.setScreenId(""+screenId);
    	screenRequest.setAction(action);
    	return screenRequest ;
    }
    
    
	//-------------------------------------------------------------------------------------
    /**
     * Returns the 'fake http session' associated with the test manager instance
     * @return
     */
    public HttpSession getHttpSession()
    {
    	return sessionManager.getSession();
    }
    
    //-------------------------------------------------------------------------------------
    /**
    private int getIntScreenId(String sScreenId) throws TelosysException
    {

        //--- Get the Screen Id
        int iScreenId = 0;
        try
        {
            iScreenId = Integer.parseInt(sScreenId);
        } catch (Exception e)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " ) ! ");
        }
        if (iScreenId < 0)
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " < 0 ) ! ");
        }
        if (iScreenId > ScreenSession.getMaxScreenId())
        {
            throw new TelosysException("ScreenRequestProcessor.openNewScreen : Invalid screen Id ( " + sScreenId + " > " + ScreenSession.getMaxScreenId() + " ) ! ");
        }
        return iScreenId;
    }
	**/
    //-------------------------------------------------------------------------------------
    private ScreenResponse getScreenResponse( ScreenRequest request ) throws TelosysException
    {
        if (request == null)
        {
            throw new TelosysException("ScreenRequestProcessor.getScreenResponse : ScreenRequest parameter is null ! ");
        }

        //---------------------------------------------------------------------------
        //--- Create a Screen Response
        //---------------------------------------------------------------------------
        String sScreenName = request.getScreenName();
        String sScreenId = request.getScreenId();
        String sAction = request.getAction();
        String sProcName = request.getProcName();

        HttpServletResponse httpResponse = new FakeHttpServletResponse() ;
        
        ScreenResponse response = new ScreenResponse(request.getHttpRequest(), httpResponse, sScreenName, sScreenId, sAction, sProcName);
                
        return response;
    }
    //-------------------------------------------------------------------------------------
//	public static ScreenContext getScreenContext( ScreenSession screenSession )
//	{
//		ScreenContext screenContext = null ;
//		
//		try {
//			screenContext = ScreenContextManager.openScreenContext(screenSession, "", 1 ); // TODO
//		} catch (TelosysException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//		
//		return screenContext ;
//	}
	
    //-------------------------------------------------------------------------------------
    /***
	private ScreenContext getScreenContext( ScreenRequest screenRequest ) throws TelosysException
	{
		ScreenContext screenContext = null ;

		ScreenSession screenSession = screenRequest.getScreenSession();
		String screenName = screenRequest.getScreenName() ;
		int screenId = getIntScreenId( screenRequest.getScreenId() ) ;
		
		try {
			screenContext = ScreenContextManager.openScreenContext(screenSession, screenName, screenId ); 
		} catch (TelosysException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return screenContext ;
	}
	***/
    //-------------------------------------------------------------------------------------
	/***
    private DatabaseSession getDatabaseSession( ScreenManager screenManager ) throws TelosysException
    {
        int iDbId = screenManager.getDatabaseId();
        return DatabaseSessionManager.getThreadLocalDatabaseSession(iDbId);
    }
	***/
	
    //-------------------------------------------------------------------------------------
    /***
    public ScreenContext actionLoad( ScreenManager screenManager, ScreenRequest screenRequest ) throws TelosysException
	{
    	System.out.println("-- actionLoad()...");
//		AuthorData screenData = new AuthorData();
//		
//		AuthorMgr screenManager = new AuthorMgr();
		
		//DatabaseSession dbSession = getDatabaseSession( screenManager );
				
		ScreenResponse screenResponse = getScreenResponse(screenRequest);
		//ScreenSession screenSession = screenRequest.getScreenSession() ;
		//ScreenSession screenSession = getScreenSession();
		
		ScreenContext screenContext = CopyOfScreenRequestProcessor.getScreenContext(screenRequest, screenRequest.getScreenSession() ) ;
		//ScreenContext screenContext = getScreenContext(screenRequest);
		
//		//screenContext.g
//		screenManager.
//		screenManager.load(screenData, screenContext, screenRequest, screenResponse, dbSession);
		
		StandardScreenActions actions = new StandardScreenActions( screenManager, null, null );
		
		actions.actionLoad(screenContext, screenRequest, screenResponse);
		
    	return screenContext ;
	}
	***/
    //-------------------------------------------------------------------------------------
    //public ScreenContext executeAction( ScreenRequest screenRequest ) throws TelosysException
    /**
     * Executes the action defined in the given ScreenRequest 
     * @param screenRequest
     * @return
     * @throws TelosysException
     */
    public ActionObjects executeAction( ScreenRequest screenRequest ) throws TelosysException
	{
    	System.out.println("-- executeAction()...");

    	ScreenResponse screenResponse = getScreenResponse(screenRequest);
		ScreenSession  screenSession = screenRequest.getScreenSession() ;
		
		ScreenContext screenContext = CopyOfScreenRequestProcessor.getScreenContext( screenRequest, screenSession ) ;

		CopyOfScreenRequestProcessor.executeAction(screenSession, screenContext, screenRequest, screenResponse);
		
    	//return screenContext ;
		
		ActionObjects objects = new ActionObjects(screenRequest, screenResponse, screenSession, screenContext );
    	return objects ;
	}
		
    //-------------------------------------------------------------------------------------
    private FakeHttpSessionManager initSessionManager() 
	{
    	System.out.println("-- initSessionManager()...");
    	FakeHttpSessionManager sessionManager = new FakeHttpSessionManager();
    	
		HttpServletRequest tmpHttpServletRequest = new FakeHttpServletRequest(sessionManager);
//		LoginUser loginUser = null ;
//		IAppUser appUser = null ;
    	LoginUser loginUser = Auth.createDummyUser(tmpHttpServletRequest);
    	IAppUser appUser = new FakeAppUser();
		ScreenSessionManager.openScreenSession( tmpHttpServletRequest, loginUser, appUser);
    	
		return sessionManager ;
	}
    
}
